<?php

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Ignore reCaptcha on confirmation screen
 *
 */
function wpbs_validate_recaptcha_payment_confirmation_redsys($response, $form_data)
{
    if (isset($form_data['wpbs-redsys-confirmation-loaded']) && $form_data['wpbs-redsys-confirmation-loaded'] == '1') {
        return true;
    }
    return $response;
}
add_filter('wpbs_validate_recaptcha_payment_confirmation', 'wpbs_validate_recaptcha_payment_confirmation_redsys', 10, 2);

/**
 * Show the payment confirmation page after submitting the form
 *
 */
function wpbs_redsys_submit_form_payment_confirmation($response, $post_data, $form, $form_args, $form_fields, $calendar_id)
{
    // Check if another payment method was already found
    if ($response !== false) {
        return $response;
    }

    $payment_found = false;

    // Check if payment method is enabled.
    foreach ($form_fields as $form_field) {
        if ($form_field['type'] == 'payment_method' && ($form_field['user_value'] == 'redsys' || $form_field['user_value'] == 'redsys_bizum')) {
            $payment_found = true;
            $payment_method = $form_field['user_value'];
            break;
        }
    }

    if ($payment_found === false) {
        return false;
    }

    // Include REDSYS API
    include_once WPBS_REDSYS_PLUGIN_DIR . 'includes/libs/redsys-api/redsys-api.php';

    $api = WPBS_Redsys_API::keys();

    // Check for API Keys
    if (empty($api['merchant_code']) || empty($api['api_key'])) {
        return json_encode(
            array(
                'success' => false,
                'html' => '<p class="wpbs-form-general-error">' . __("Please add your API keys in the plugin's Settings Page.", 'wp-booking-system-redsys') . '</p>',
            )
        );
    }

    // Parse POST data
    parse_str($post_data['form_data'], $form_data);

    // Check if the payment screen was shown
    if (isset($form_data['wpbs-redsys-confirmation-loaded']) && $form_data['wpbs-redsys-confirmation-loaded'] == '1') {
        return false;
    }

    // Generate form
    $form_outputter = new WPBS_Form_Outputter($form, $form_args, $form_fields, $calendar_id);

    // Check if post data exists and matches form values
    if (wpbs_validate_payment_form_consistency($form_fields) === false) {
        return json_encode(
            array(
                'success' => false,
                'html' => '<strong>' . __('Something went wrong. Please refresh the page and try again.', 'wp-booking-system-redsys') . '</strong>',
            )
        );
    }

    // Get price
    $payment = new WPBS_Payment;
    $payment->calculate_prices($post_data, $form, $form_args, $form_fields);

    // Only EUR transactions should be allowed
    if ($payment->get_currency() != 'EUR') {
        return json_encode(
            array(
                'success' => false,
                'html' => '<p class="wpbs-form-general-error">' . __('Redsys only allows EUR transactions.', 'wp-booking-system-redsys') . '</p>',
            )
        );
    }

    $total = $payment->get_total();

    // Check if part payments are used
    if (wpbs_part_payments_enabled() == true && $payment->is_part_payment()) {
        $total = $payment->get_total_first_payment();
    }

    // Check if price is greater than the minimum allowed, 0.5;
    if ($total <= 0.5) {
        return json_encode(
            array(
                'success' => false,
                'html' => '<p class="wpbs-form-general-error">' . __("The minimum payable amount is 0.50$", 'wp-booking-system-redsys') . '</p>',
            )
        );
    }

    /**
     * Create the Payment Intent on Redsys
     *
     */
    $booking_data = array(
        'post_data' => $post_data,
        'form_id' => $form->get('id'),
        'form_args' => $form_args,
        'form_fields' => $form_fields,
        'calendar_id' => $calendar_id,
        'payment_method' => $payment_method
    );

    // Generate an unique ID for the transient id
    $booking_data_transient_id = substr(uniqid(), 1, 12);

    // Save all the booking details for later use, if the payment is accepted
    set_transient($booking_data_transient_id, $booking_data, DAY_IN_SECONDS * 7);

    $return_url = $form_data['wpbs-return-url'] . '?wpbs-redsys';
    $remote_payment_method = $payment_method == 'redsys_bizum' ? 'z' : '';

    $redsys = new WPBS_Redsys_Payment();
    $redsys_payment = $redsys->createPaymentRequest($total * 100, $booking_data_transient_id, $return_url, $remote_payment_method);

    /**
     * Prepare Response
     *
     */
    $redsys_output = '';
    if ($api['production_mode'] == false) {
        $redsys_output .= '<p class="wpbs-payment-test-mode-enabled">' . __('Redsys Test mode is enabled.', 'wp-booking-system-redsys') . '</p>';
    }

    $redsys_output .= '<div class="wpbs-payment-confirmation-redsys-form">';

    if (wpbs_part_payments_enabled() == true && $payment->is_part_payment()) {
        $redsys_output .= '<label>' . wpbs_get_payment_default_string('amount_billed', $form_outputter->get_language()) . '</label><input class="wpbs-payment-confirmation-redsys-input" type="text" value="' . wpbs_get_formatted_price($total, $payment->get_currency()) . '" readonly>';
    }

    $redsys_output .= $redsys_payment->form(false, wpbs_get_payment_default_string('payment_submit', $form_outputter->get_language()));
    $redsys_output .= '<div class="wpbs-redsys-powered-by"><img src="' . WPBS_REDSYS_PLUGIN_DIR_URL . 'assets/img/redsys-logo.png"></div>';

    $redsys_output .= '</div>';

    $redsys_output .= '
    <script>
        jQuery(".wpbs-redsys-payment-confirmation-inner-' . $form_outputter->get_unique() . '").parents(".wpbs-main-wrapper").find(".wpbs-container").addClass("wpbs-disable-selection");
    </script>
    ';

    $output = wpbs_form_payment_confirmation_screen($form_outputter, $payment, 'redsys', $redsys_output);

    return json_encode(
        array(
            'success' => false,
            'html' => $output,
        )
    );

}
add_filter('wpbs_submit_form_before', 'wpbs_redsys_submit_form_payment_confirmation', 10, 6);

/**
 * Show a message when the payment failed.
 *
 */
function wpbs_redsys_form_outputter_payment_failed()
{
    // Check if we're on the redsys confirmation page
    if (!isset($_GET['wpbs-redsys'])) {
        return false;
    }

    if (!isset($_GET['wpbs_redsys_payment_failed'])) {
        return false;
    }

    // Disable date selection on calendar
    add_filter('wpbs_calendar_outputter_custom_class', function () {
        return 'wpbs-disable-selection wpbs-scroll-to-calendar';
    });

    // Replace the form with the confirmation message
    add_filter('wpbs_form_outputter_form', function ($output) {
        return '
        <script>jQuery(document).ready(function(){
            if(jQuery("#wpbs-payment-error-message").length){
                jQuery("html, body").scrollTop(jQuery("#wpbs-payment-error-message").offset().top);
            }
        });</script>
        <p class="wpbs-payment-error-message" id="wpbs-payment-error-message">The payment was declined. Your credit card was not charged. <a href="' . get_permalink() . '">Please try again.</a></p>';
    });

}
add_action('init', 'wpbs_redsys_form_outputter_payment_failed', 20);

/**
 * Show a message when the payment succeeded.
 *
 */
function wpbs_redsys_form_outputter_payment_succeeded()
{
    // Check if we're on the redsys confirmation page
    if (!isset($_GET['wpbs-redsys'])) {
        return false;
    }

    if (!isset($_GET['wpbs_redsys_payment_succeeded'])) {
        return false;
    }

    if (!isset($_GET['wpbs_redsys_order_id'])) {
        return false;
    }

    global $response;
    $response = get_transient($_GET['wpbs_redsys_order_id'] . '_response');

    if (!$response) {
        add_filter('wpbs_form_outputter_form', function ($output) {
            $confirmation = '<script>jQuery(document).ready(function(){
                if(jQuery("#wpbs-payment-success-message").length){
                    jQuery("html, body").scrollTop(jQuery("#wpbs-payment-success-message").offset().top);
                }
                setTimeout(function(){
                    window.location.reload();
                }, 3000);
            });</script>
            <div id="wpbs-payment-success-message">
            <p>'.wpbs_get_payment_default_string('processing_payment', wpbs_get_locale()).'</p>
            </div>';

            return $confirmation;
        });
        return false;
    }

    if(isset($response['received'])){
        return false;
    }

    $response['received'] = true;
    set_transient($_GET['wpbs_redsys_order_id'] . '_response', $response, DAY_IN_SECONDS);

    // Disable date selection on calendar
    add_filter('wpbs_calendar_outputter_custom_class', function () {
        return 'wpbs-disable-selection wpbs-scroll-to-calendar';
    });

    

    // Replace the form with the confirmation message
    add_filter('wpbs_form_outputter_form', function ($output) {
        global $response;
        $confirmation = '<script>jQuery(document).ready(function(){
            if(jQuery("#wpbs-payment-success-message").length){
                jQuery("html, body").scrollTop(jQuery("#wpbs-payment-success-message").offset().top);
            }
        });</script>
        <div id="wpbs-payment-success-message">';

        if ($response['tracking_script']) {
            $confirmation .= '<script>' . $response['tracking_script'] . '</script>';
        }

        if ($response['confirmation_type'] == 'message') {
            $confirmation .= '<div class="wpbs-form-confirmation-message">' . $response['confirmation_message'] . '</div>';
        }

        if ($response['confirmation_type'] == 'redirect') {
            $confirmation .= '<script> window.location.href = "' . $response['confirmation_redirect_url'] . '"</script>';
        }

        $confirmation .= '</div>';

        return $confirmation;
    });

}
add_action('init', 'wpbs_redsys_form_outputter_payment_succeeded', 20);

/**
 * Webhook used to save the order and create the booking.
 *
 */
function wpbs_redsys_webhook()
{

    // Check if we're on the redsys confirmation page
    if (!isset($_GET['wpbs-redsys'])) {
        return false;
    }

    if (!isset($_GET['wpbs_redsys_payment_webhook'])) {
        return false;
    }

    // Include Redsys API
    include_once WPBS_REDSYS_PLUGIN_DIR . 'includes/libs/redsys-api/redsys-api.php';

    // Get Payment Details
    $redsys_payment = new WPBS_Redsys_Payment();
    $payment_data = $redsys_payment->getPaymentData();

    if (!isset($payment_data['status'])) {
        return false;
    }

    if ($payment_data['status'] != 'succeeded') {
        return false;
    }

    // Exit if no additional parameters were sent.
    if (!isset($payment_data['order'])) {
        return false;
    }

    // Get booking data from transient
    $booking_data = get_transient($payment_data['order']);

    // Check if booking data exists
    if (empty($booking_data)) {
        return false;
    }

    $payment_data['booking_data'] = $booking_data;

    // Delete the transient to prevent double bookings
    delete_transient($payment_data['order']);

    // Add the payment ID to the $post_data
    $booking_data['post_data']['wpbs-redsys-payment-id'] = $payment_data['order'];
    $booking_data['post_data']['wpbs-redsys-payment-data'] = $payment_data;

    // Finally submit the form, saving booking details, sending emails, etc
    $handler = new WPBS_Form_Handler($booking_data['post_data'], $booking_data['form_id'], $booking_data['form_args'], $booking_data['form_fields'], $booking_data['calendar_id']);
    $response = $handler->get_response();

    // Save response for later
    set_transient($payment_data['order'] . '_response', $response, DAY_IN_SECONDS * 7);

}
add_action('init', 'wpbs_redsys_webhook', 20);

/**
 * Save the order in the database and maybe capture the payment
 *
 */
function wpbs_redsys_action_save_payment_details($booking_id, $post_data, $form, $form_args, $form_fields)
{

    // Parse POST data
    parse_str($post_data['form_data'], $form_data);

    // Check if redsys is enabled
    if (!isset($post_data['wpbs-redsys-payment-id'])) {
        return false;
    }

    if (!isset($post_data['wpbs-redsys-payment-data'])) {
        return false;
    }

    // Get price
    $payment = new WPBS_Payment;
    $details['price'] = $payment->calculate_prices($post_data, $form, $form_args, $form_fields);

    if (wpbs_part_payments_enabled() == true && $payment->is_part_payment()) {
        $details['part_payments'] = array('deposit' => false, 'final_payment' => false);
    }

    $payment_data = $post_data['wpbs-redsys-payment-data'];

    if ($payment_data['status'] == 'succeeded') {
        $details['raw'] = $payment_data;

        if (isset($details['part_payments']['deposit'])) {
            $details['part_payments']['deposit'] = true;
        }

        $status = 'completed';
        $id = $payment_data['authorisation_code'];
    } else {
        $details['error'] = $payment_data['error_code'];
        $status = 'error';
        $id = 'N/A';
    }

    // Save Order
    wpbs_insert_payment(array(
        'booking_id' => $booking_id,
        'gateway' => $payment_data['booking_data']['payment_method'],
        'order_id' => $id,
        'order_status' => $status,
        'details' => $details,
        'date_created' => current_time('Y-m-d H:i:s'),
    ));

}
add_action('wpbs_submit_form_after', 'wpbs_redsys_action_save_payment_details', 10, 5);
